@@ -1474,6 +1474,10 @@ class CursesApplication(Application):
             if panel not in [self.tasks, self.search_bar]:
                 panel.paint()
 
+    def refresh(self, force=False):
+        super().refresh(force)
+        self.paint()
+
     def resize(self):
         height, width = self.size()
 
