@@ -21,6 +21,7 @@ class Searcher:
         self.done = None
         self.show_hidden = False
         self.priority = None
+        self.not_priority = False
         self.default_threshold = default_threshold
         self.hide_sequential = hide_sequential
         if default_threshold is None or len(default_threshold) == 0:
@@ -32,6 +33,7 @@ class Searcher:
         self.casesensitive = casesensitive
         self.sources = None
         self.task_by_id = None
+        self.parents = None
 
         self.text = text
 
@@ -54,7 +56,8 @@ class Searcher:
         self.done = None
         self.show_hidden = False
         self.priority = None
-        self.threshold = get_relative_date(self.default_threshold, Task.DATE_FMT) or self.default_threshold
+        self.threshold = get_relative_date(self.default_threshold, Task.DATE_FMT) or \
+                         self.default_threshold
         self.due = None
         self.created = None
         self.completed = None
@@ -99,14 +102,16 @@ class Searcher:
     def _parse_ids(self, keyword, task):
         these = set()
         for value in task.attributes.get(keyword, []):
-            these |= {that for that in value.split(',')}
+            these |= set(value.split(','))
         if self.casesensitive:
             return {that.lower() for that in these}
-        else:
-            return these
+        return these
 
     def parse(self):
         self.reset()
+        if self.text is None:
+            return
+
         text = self.text
         if not self.casesensitive:
             text = text.lower()
@@ -132,92 +137,107 @@ class Searcher:
                     self.not_contexts.add(part[1:])
                 else:
                     self.contexts.add(part[1:])
-            
+
             elif part.startswith('+') and len(part) > 1:
                 if do_not:
                     self.not_projects.add(part[1:])
                 else:
                     self.projects.add(part[1:])
-            
+
             elif part.startswith('done:'):
                 _, value = part.split(':', 1)
                 if len(value) == 0:
                     self.done = None
                 else:
                     self.done = value.lower().startswith('y')
-            
+
             elif part.startswith('hidden:') or part.startswith('h:'):
                 _, value = part.split(':', 1)
                 self.show_hidden = value.lower().startswith('y') or value == '1'
-            
+
             elif part.startswith('pri:'):
                 _, value = part.split(':', 1)
                 self.priority = (value, value)
-            
+                if do_not:
+                    self.not_priority = True
+
             elif part.startswith('moreimportant:') or part.startswith('mi:'):
                 _, value = part.split(':', 1)
-                if self.priority is None:
-                    self.priority = ['ZZZZ', ' ']
-                self.priority[0] = value.upper()
-            
+                if not isinstance(self.priority, list):
+                    # upper bound, lower bound
+                    self.priority = [' ', 'ZZZZ']
+                self.priority[1] = value.upper()
+                if do_not:
+                    self.not_priority = True
+
             elif part.startswith('lessimportant:') or part.startswith('li:'):
                 _, value = part.split(':', 1)
-                if self.priority is None:
-                    self.priority = ['ZZZZ', ' ']
-                self.priority[1] = value.upper()
-            
+                if not isinstance(self.priority, list):
+                    # upper bound, lower bound
+                    self.priority = [' ', 'ZZZZ']
+                self.priority[0] = value.upper()
+                if do_not:
+                    self.not_priority = True
+
             elif part.startswith('due:'):
                 _, value = part.split(':', 1)
-                if value.lower().startswith('y'):
+                if value.lower() in ['y', 'yes']:
                     self.due = ['0000-00-00', '9999-99-99']
+                elif value.lower() == 'no':
+                    self.due = 'no'
                 else:
+                    value = get_relative_date(value, Task.DATE_FMT) or value
                     self.due = [value, value]
-            
+
             elif part.startswith('duebefore:') or part.startswith('db:'):
                 _, value = part.split(':', 1)
-                if self.due is None:
+                if not isinstance(self.due, list):
                     self.due = ['0000-00-00', '9999-99-99']
-                self.due[1] = value
-            
+                self.due[1] = get_relative_date(value, Task.DATE_FMT) or value
+
             elif part.startswith('dueafter:') or part.startswith('da:'):
                 _, value = part.split(':', 1)
-                if self.due is None:
+                if not isinstance(self.due, list):
                     self.due = ['0000-00-00', '9999-99-99']
-                self.due[0] = value
+                self.due[0] = get_relative_date(value, Task.DATE_FMT) or value
 
             elif part.startswith('created:'):
                 _, value = part.split(':', 1)
+                value = get_relative_date(value, Task.DATE_FMT) or value
                 self.created = [value, value]
 
             elif part.startswith('createdbefore:') or part.startswith('crb:'):
                 _, value = part.split(':', 1)
                 if self.created is None:
                     self.created = ['0000-00-00', '9999-99-99']
-                self.created[1] = value
+                self.created[1] = get_relative_date(value, Task.DATE_FMT) or value
 
             elif part.startswith('createdafter:') or part.startswith('cra:'):
                 _, value = part.split(':', 1)
                 if self.created is None:
                     self.created = ['0000-00-00', '9999-99-99']
-                self.created[0] = value
+                self.created[0] = get_relative_date(value, Task.DATE_FMT) or value
 
             elif part.startswith('completed:'):
                 _, value = part.split(':', 1)
+                value = get_relative_date(value, Task.DATE_FMT) or value
                 self.completed = [value, value]
 
             elif part.startswith('completedbefore:') or part.startswith('cob:'):
                 _, value = part.split(':', 1)
                 if self.completed is None:
                     self.completed = ['0000-00-00', '9999-99-99']
-                self.completed[1] = value
+                self.completed[1] = get_relative_date(value, Task.DATE_FMT) or value
 
             elif part.startswith('completedafter:') or part.startswith('coa:'):
                 _, value = part.split(':', 1)
                 if self.completed is None:
                     self.completed = ['0000-00-00', '9999-99-99']
-                self.completed[0] = value
+                self.completed[0] = get_relative_date(value, Task.DATE_FMT) or value
 
-            elif part.startswith('t:') or part.startswith('tickler:') or part.startswith('threshold:'):
+            elif part.startswith('t:') or \
+                 part.startswith('tickler:') or \
+                 part.startswith('threshold:'):
                 _, value = part.split(':', 1)
                 if len(value) == 0:
                     self.threshold = None
@@ -227,7 +247,7 @@ class Searcher:
             elif part.startswith('id:'):
                 _, value = part.split(':', 1)
                 if len(value) > 0:
-                    value = {that for that in value.split(',')}
+                    value = set(value.split(','))
 
                     if do_not:
                         self.not_ids |= value
@@ -266,7 +286,7 @@ class Searcher:
                         self.not_filenames.add(value)
                     else:
                         self.filenames.add(value)
-            
+
             else:
                 if do_not:
                     self.not_words.add(part)
@@ -338,13 +358,21 @@ class Searcher:
             return True
 
         pri = 'ZZZ'
+        upper, lower = self.priority
         if task.priority is not None:
             pri = task.priority
         if not self.casesensitive:
             pri = pri.lower()
+            lower = lower.lower()
+            upper = upper.lower()
 
-        return (self.priority[0] == self.priority[1] and pri == self.priority[0]) or \
-               self.priority[0] > pri > self.priority[1]
+        # upper < pri < lower is counter-intuitive, but:
+        # pri 'a' < pri 'b'
+        result = (lower == upper and pri == lower) or \
+                 upper < pri < lower
+        if self.not_priority:
+            return not result
+        return result
 
     def match_threshold(self, attrs):
         if self.threshold is None:
@@ -355,16 +383,16 @@ class Searcher:
         if self.due is None:
             return True
 
+        if self.due == 'no':
+            return 'due' not in attrs
+
         if 'due' not in attrs:
             return False
 
-        due = [get_relative_date(self.due[0], Task.DATE_FMT) or self.due[0],
-               get_relative_date(self.due[1], Task.DATE_FMT) or self.due[1]]
+        if self.due[0] == self.due[1]:
+            return attrs['due'][0] == self.due[0]
 
-        if due[0] == due[1]:
-            return attrs['due'][0] == due[0]
-
-        return due[0] < attrs['due'][0] < due[1]
+        return self.due[0] < attrs['due'][0] < self.due[1]
 
     def match_created(self, task):
         if self.created is None:
@@ -373,14 +401,12 @@ class Searcher:
         if task.creation_date is None:
             return False
 
-        created = [get_relative_date(self.created[0], Task.DATE_FMT) or self.created[0],
-                   get_relative_date(self.created[1], Task.DATE_FMT) or self.created[1]]
         task_created = task.creation_date.strftime(Task.DATE_FMT)
 
-        if created[0] == created[1]:
-            return created[0] == task_created
+        if self.created[0] == self.created[1]:
+            return self.created[0] == task_created
 
-        return created[0] < task_created < created[1]
+        return self.created[0] < task_created < self.created[1]
 
     def match_completed(self, task):
         if self.completed is None:
@@ -389,14 +415,12 @@ class Searcher:
         if task.completion_date is None:
             return False
 
-        completed = [get_relative_date(self.completed[0], Task.DATE_FMT) or self.completed[0],
-                     get_relative_date(self.completed[1], Task.DATE_FMT) or self.completed[1]]
         task_completed = task.completion_date.strftime(Task.DATE_FMT)
 
-        if completed[0] == completed[1]:
-            return completed[0] == task_completed
+        if self.completed[0] == self.completed[1]:
+            return self.completed[0] == task_completed
 
-        return completed[0] < task_completed < completed[1]
+        return self.completed[0] < task_completed < self.completed[1]
 
     def match_ids(self, task):
         if len(self.not_ids) == 0 and len(self.ids) == 0:
@@ -470,6 +494,8 @@ def get_relative_date(text, fmt=None, base=None):
         then = today
     if text.lower() == 'tomorrow':
         then = today + datetime.timedelta(days=1)
+    if text.lower() == 'yesterday':
+        then = today - datetime.timedelta(days=1)
     if text.lower() in ['next-week', 'week']:
         then = today + datetime.timedelta(weeks=1)
     if text.lower() in weekdays:
